@@ -36,6 +36,8 @@
 
 #include <dbus/dbus.h>
 
+#include <spa/debug/mem.h>
+#include <spa/debug/log.h>
 #include <spa/support/log.h>
 #include <spa/support/loop.h>
 #include <spa/support/dbus.h>
@@ -74,7 +76,7 @@ enum {
 	HFP_AG_INITIAL_CODEC_SETUP_WAIT
 };
 
-#define CIND_INDICATORS "(\"service\",(0-1)),(\"call\",(0-1)),(\"callsetup\",(0-3)),(\"callheld\",(0-2)),(\"signal\",(0-5)),(\"roam\",(0-1)),\"battchg\",(0-5))"
+#define CIND_INDICATORS "(\"service\",(0-1)),(\"call\",(0-1)),(\"callsetup\",(0-3)),(\"callheld\",(0-2)),(\"signal\",(0-5)),(\"roam\",(0-1)),(\"battchg\",(0-5))"
 enum {
 	CIND_SERVICE = 1,
 	CIND_CALL,
@@ -779,6 +781,12 @@ static bool rfcomm_hfp_ag(struct rfcomm *rfcomm, char* buf)
 	int xapl_product;
 	int xapl_features;
 
+	spa_debug_log_mem(backend->log, SPA_LOG_LEVEL_DEBUG, 2, buf, strlen(buf));
+
+	/* Some devices send initial \n: be permissive */
+	while (*buf == '\n')
+		++buf;
+
 	if (sscanf(buf, "AT+BRSF=%u", &features) == 1) {
 		unsigned int ag_features = SPA_BT_HFP_AG_FEATURE_NONE;
 
@@ -880,6 +888,9 @@ static bool rfcomm_hfp_ag(struct rfcomm *rfcomm, char* buf)
 				rfcomm_emit_volume_changed(rfcomm, -1, SPA_BT_VOLUME_INVALID);
 			}
 		}
+	} else if (spa_streq(buf, "\r")) {
+		/* No commands, reply OK (ITU-T Rec. V.250 Sec. 5.2.1 & 5.6) */
+		rfcomm_send_reply(rfcomm, "OK");
 	} else if (!rfcomm->slc_configured) {
 		spa_log_warn(backend->log, "RFCOMM receive command before SLC completed: %s", buf);
 		rfcomm_send_error(rfcomm, CMEE_AG_FAILURE);
@@ -1111,10 +1122,10 @@ next_indicator:
 			return true;
 		}
 	} else if (spa_strstartswith(buf, "ATD")) {
-		char number[30];
+		char number[31], sep;
 		enum cmee_error error;
 
-		if (sscanf(buf, "ATD%30s;", number) != 1) {
+		if (sscanf(buf, "ATD%30[^;]%c", number, &sep) != 2 || sep != ';') {
 			spa_log_debug(backend->log, "Failed to parse ATD: \"%s\"", buf);
 			rfcomm_send_error(rfcomm, CMEE_AG_FAILURE);
 			return true;
