@@ -48,6 +48,8 @@
 #include <spa/param/audio/format.h>
 #include <spa/param/audio/format-utils.h>
 #include <spa/pod/filter.h>
+#include <spa/debug/mem.h>
+#include <spa/debug/log.h>
 
 #include <sbc/sbc.h>
 
@@ -839,11 +841,15 @@ static void media_on_flush_timeout(struct spa_source *source)
 {
 	struct impl *this = source->data;
 	uint64_t exp;
+	int res;
 
 	spa_log_trace(this->log, "%p: flush on timeout", this);
 
-	if (spa_system_timerfd_read(this->data_system, this->flush_timerfd, &exp) < 0)
-		spa_log_warn(this->log, "error reading timerfd: %s", strerror(errno));
+	if ((res = spa_system_timerfd_read(this->data_system, this->flush_timerfd, &exp)) < 0) {
+		if (res != -EAGAIN)
+			spa_log_warn(this->log, "error reading timerfd: %s", spa_strerror(res));
+		return;
+	}
 
 	if (this->transport == NULL) {
 		enable_flush_timer(this, false);
@@ -864,12 +870,19 @@ static void media_on_timeout(struct spa_source *source)
 	uint32_t rate;
 	struct spa_io_buffers *io = port->io;
 	uint64_t prev_time, now_time;
+	int res;
 
 	if (this->transport == NULL)
 		return;
 
-	if (this->started && spa_system_timerfd_read(this->data_system, this->timerfd, &exp) < 0)
-		spa_log_warn(this->log, "error reading timerfd: %s", strerror(errno));
+	if (this->started) {
+		if ((res = spa_system_timerfd_read(this->data_system, this->timerfd, &exp)) < 0) {
+			if (res != -EAGAIN)
+				spa_log_warn(this->log, "error reading timerfd: %s",
+						spa_strerror(res));
+			return;
+		}
+	}
 
 	prev_time = this->current_time;
 	now_time = this->current_time = this->next_time;
@@ -938,7 +951,7 @@ static int do_start(struct impl *this)
 	size = this->transport->configuration_len;
 
 	spa_log_debug(this->log, "Transport configuration:");
-	spa_log_hexdump(this->log, SPA_LOG_LEVEL_DEBUG, 2, conf, (size_t)size);
+	spa_debug_log_mem(this->log, SPA_LOG_LEVEL_DEBUG, 2, conf, (size_t)size);
 
 	flags = this->is_duplex ? MEDIA_CODEC_FLAG_SINK : 0;
 
@@ -1442,11 +1455,13 @@ impl_node_port_use_buffers(void *object,
 
 	spa_log_debug(this->log, "use buffers %d", n_buffers);
 
-	if (!port->have_format)
-		return -EIO;
-
 	clear_buffers(this, port);
 
+	if (n_buffers > 0 && !port->have_format)
+		return -EIO;
+	if (n_buffers > MAX_BUFFERS)
+		return -ENOSPC;
+
 	for (i = 0; i < n_buffers; i++) {
 		struct buffer *b = &port->buffers[i];
 
@@ -1743,7 +1758,6 @@ impl_init(const struct spa_handle_factory *factory,
 	port->latency = SPA_LATENCY_INFO(SPA_DIRECTION_INPUT);
 	port->latency.min_quantum = 1.0f;
 	port->latency.max_quantum = 1.0f;
-	set_latency(this, false);
 
 	spa_list_init(&port->ready);
 
@@ -1789,6 +1803,8 @@ impl_init(const struct spa_handle_factory *factory,
 
 	reset_props(this, &this->props);
 
+	set_latency(this, false);
+
 	spa_bt_transport_add_listener(this->transport,
 			&this->transport_listener, &transport_events, this);
 
