@@ -35,6 +35,8 @@ extern const struct spa_handle_factory spa_sco_sink_factory;
 extern const struct spa_handle_factory spa_sco_source_factory;
 extern const struct spa_handle_factory spa_a2dp_sink_factory;
 extern const struct spa_handle_factory spa_a2dp_source_factory;
+extern const struct spa_handle_factory spa_bluez5_midi_enum_factory;
+extern const struct spa_handle_factory spa_bluez5_midi_node_factory;
 
 SPA_EXPORT
 int spa_handle_factory_enum(const struct spa_handle_factory **factory, uint32_t *index)
@@ -67,6 +69,12 @@ int spa_handle_factory_enum(const struct spa_handle_factory **factory, uint32_t
 	case 7:
 		*factory = &spa_a2dp_source_factory;
 		break;
+	case 8:
+		*factory = &spa_bluez5_midi_enum_factory;
+		break;
+	case 9:
+		*factory = &spa_bluez5_midi_node_factory;
+		break;
 	default:
 		return 0;
 	}
