@@ -44,7 +44,7 @@
 #include <spa/pod/builder.h>
 #include <spa/monitor/device.h>
 #include <spa/monitor/utils.h>
-#include <spa/debug/pod.h>
+#include <spa/param/param.h>
 
 #include "libcamera.h"
 #include "libcamera-manager.hpp"
