@@ -744,7 +744,7 @@ do_port_use_buffers(struct impl *impl,
 	struct node *this = &impl->node;
 	struct port *p;
 	struct mix *mix;
-	uint32_t i, j, peer_id;
+	uint32_t i, j;
 	struct pw_client_node_buffer *mb;
 
 	p = GET_PORT(this, direction, port_id);
@@ -754,7 +754,6 @@ do_port_use_buffers(struct impl *impl,
 	if (n_buffers > MAX_BUFFERS)
 		return -ENOSPC;
 
-
 	spa_log_debug(this->log, "%p: %s port %d.%d use buffers %p %u flags:%08x", this,
 			direction == SPA_DIRECTION_INPUT ? "input" : "output",
 			port_id, mix_id, buffers, n_buffers, flags);
@@ -762,8 +761,6 @@ do_port_use_buffers(struct impl *impl,
 	if ((mix = find_mix(p, mix_id)) == NULL || !mix->valid)
 		return -EINVAL;
 
-	peer_id = mix->peer_id;
-
 	if (direction == SPA_DIRECTION_OUTPUT) {
 		mix_id = SPA_ID_INVALID;
 		if ((mix = find_mix(p, mix_id)) == NULL || !mix->valid)
@@ -809,7 +806,7 @@ do_port_use_buffers(struct impl *impl,
 			endptr = SPA_PTROFF(endptr, SPA_ROUND_UP_N(buffers[i]->metas[j].size, 8), void);
 		}
 		for (j = 0; j < buffers[i]->n_datas; j++) {
-			struct spa_data *d = buffers[i]->datas;
+			struct spa_data *d = &buffers[i]->datas[j];
 			if (d->type == SPA_DATA_MemPtr) {
 				if ((m = pw_mempool_find_ptr(impl->context->pool, d->data)) == NULL ||
 				    m != mem)
@@ -881,10 +878,6 @@ do_port_use_buffers(struct impl *impl,
 	}
 	mix->n_buffers = n_buffers;
 
-	if (this->resource->version >= 4)
-		pw_client_node_resource_port_set_mix_info(this->resource,
-						 direction, port_id, mix_id,
-						 peer_id, NULL);
 	return pw_client_node_resource_port_use_buffers(this->resource,
 						 direction, port_id, mix_id, flags,
 						 n_buffers, mb);
@@ -1486,6 +1479,7 @@ static int impl_mix_port_set_io(void *object,
 	struct port *p = object;
 	struct pw_impl_port *port = p->port;
 	struct impl *impl = port->owner_data;
+	struct node *this = &impl->node;
 	struct pw_impl_port_mix *mix;
 
 	mix = pw_map_lookup(&port->mix_port_map, mix_id);
@@ -1497,6 +1491,11 @@ static int impl_mix_port_set_io(void *object,
 			mix->io = data;
 		else
 			mix->io = NULL;
+
+		if (mix->io != NULL && this->resource && this->resource->version >= 4)
+			pw_client_node_resource_port_set_mix_info(this->resource,
+						 direction, port->port_id,
+						 mix->port.port_id, mix->peer_id, NULL);
 	}
 
 	return do_port_set_io(impl,
@@ -1558,14 +1557,14 @@ static void node_port_added(void *data, struct pw_impl_port *port)
 	struct impl *impl = data;
 	struct port *p = pw_impl_port_get_user_data(port);
 
-	pw_impl_port_set_mix(port, &p->mix_node,
-			PW_IMPL_PORT_MIX_FLAG_MULTI |
-			PW_IMPL_PORT_MIX_FLAG_MIX_ONLY);
-
 	port->flags |= PW_IMPL_PORT_FLAG_NO_MIXER;
 
 	port->impl = SPA_CALLBACKS_INIT(&port_impl, p);
 	port->owner_data = impl;
+
+	pw_impl_port_set_mix(port, &p->mix_node,
+			PW_IMPL_PORT_MIX_FLAG_MULTI |
+			PW_IMPL_PORT_MIX_FLAG_MIX_ONLY);
 }
 
 static void node_port_removed(void *data, struct pw_impl_port *port)
