@@ -38,6 +38,8 @@
 #include <spa/pod/filter.h>
 #include <spa/utils/keys.h>
 
+#define PW_ENABLE_DEPRECATED
+
 #include "pipewire/pipewire.h"
 #include "pipewire/private.h"
 
@@ -1330,16 +1332,16 @@ static void convert_properties(struct pw_properties *properties)
 		const char *from, *to;
 	} props[] = {
 		{ "pipewire.autoconnect", PW_KEY_NODE_AUTOCONNECT, },
+		/* XXX deprecated */
 		{ "pipewire.target.node", PW_KEY_NODE_TARGET, }
 	};
 
-	uint32_t i;
 	const char *str;
 
-	for(i = 0; i < SPA_N_ELEMENTS(props); i++) {
-		if ((str = pw_properties_get(properties, props[i].from)) != NULL) {
-			pw_properties_set(properties, props[i].to, str);
-			pw_properties_set(properties, props[i].from, NULL);
+	SPA_FOR_EACH_ELEMENT_VAR(props, p) {
+		if ((str = pw_properties_get(properties, p->from)) != NULL) {
+			pw_properties_set(properties, p->to, str);
+			pw_properties_set(properties, p->from, NULL);
 		}
 	}
 }
