@@ -33,7 +33,6 @@
 #include "defs.h"
 #include "format.h"
 #include "internal.h"
-#include "log.h"
 #include "message.h"
 #include "remap.h"
 #include "volume.h"
@@ -47,6 +46,9 @@
 
 #define PA_CHANNELS_MAX	(32u)
 
+PW_LOG_TOPIC_EXTERN(pulse_conn);
+#define PW_LOG_TOPIC_DEFAULT pulse_conn
+
 static inline uint32_t volume_from_linear(float vol)
 {
 	uint32_t v;
