@@ -26,7 +26,6 @@
 #include <spa/utils/hook.h>
 #include <spa/utils/json.h>
 #include <pipewire/pipewire.h>
-#include <pipewire/private.h>
 
 #include "../defs.h"
 #include "../module.h"
@@ -59,7 +58,7 @@ static const struct pw_impl_module_events module_events = {
 	.destroy = module_destroy
 };
 
-static int module_remap_sink_load(struct client *client, struct module *module)
+static int module_remap_sink_load(struct module *module)
 {
 	struct module_remap_sink_data *data = module->user_data;
 	FILE *f;
@@ -197,7 +196,7 @@ static int module_remap_sink_prepare(struct module * const module)
 		}
 	}
 	if ((str = pw_properties_get(props, "master")) != NULL) {
-		pw_properties_set(playback_props, PW_KEY_NODE_TARGET, str);
+		pw_properties_set(playback_props, PW_KEY_TARGET_OBJECT, str);
 		pw_properties_set(props, "master", NULL);
 	}
 
