@@ -25,7 +25,6 @@
 
 #include <spa/utils/hook.h>
 #include <pipewire/pipewire.h>
-#include <pipewire/private.h>
 
 #include "../defs.h"
 #include "../module.h"
@@ -58,7 +57,7 @@ static const struct pw_impl_module_events module_events = {
 	.destroy = module_destroy
 };
 
-static int module_roc_sink_input_load(struct client *client, struct module *module)
+static int module_roc_sink_input_load(struct module *module)
 {
 	struct module_roc_sink_input_data *data = module->user_data;
 	FILE *f;
