@@ -27,7 +27,6 @@
 #include <spa/utils/json.h>
 
 #include <pipewire/pipewire.h>
-#include <pipewire/private.h>
 #include <pipewire/i18n.h>
 
 #include "../defs.h"
@@ -62,7 +61,7 @@ static const struct pw_impl_module_events module_events = {
 	.destroy = module_destroy
 };
 
-static int module_tunnel_source_load(struct client *client, struct module *module)
+static int module_tunnel_source_load(struct module *module)
 {
 	struct module_tunnel_source_data *data = module->user_data;
 	FILE *f;
@@ -168,7 +167,7 @@ static int module_tunnel_source_prepare(struct module * const module)
 
 	remote_source_name = pw_properties_get(props, "source");
 	if (remote_source_name)
-		pw_properties_set(props, PW_KEY_NODE_TARGET, remote_source_name);
+		pw_properties_set(props, PW_KEY_TARGET_OBJECT, remote_source_name);
 
 	if ((server = pw_properties_get(props, "server")) == NULL) {
 		pw_log_error("no server given");
