@@ -24,7 +24,6 @@
 
 #include <spa/utils/hook.h>
 #include <pipewire/pipewire.h>
-#include <pipewire/private.h>
 
 #include "../defs.h"
 #include "../module.h"
@@ -57,7 +56,7 @@ static const struct pw_impl_module_events module_events = {
 	.destroy = module_destroy
 };
 
-static int module_zeroconf_discover_load(struct client *client, struct module *module)
+static int module_zeroconf_discover_load(struct module *module)
 {
 	struct module_zeroconf_discover_data *data = module->user_data;
 	FILE *f;
