@@ -191,6 +191,9 @@ static int handle_memblock(struct client *client, struct message *msg)
 
 	stream_send_request(stream);
 
+	if (stream->is_paused && !stream->corked)
+		stream_set_paused(stream, false, "new data");
+
 finish:
 	message_free(msg, false, false);
 	return res;
@@ -820,13 +823,14 @@ static int parse_ip_address(const char *address, struct sockaddr_storage *addrs,
 	if (len < 2)
 		return -ENOSPC;
 
-	snprintf(ip, sizeof(ip), "[::]:%d", res);
-	spa_assert_se(parse_ipv6_address(ip, (struct sockaddr_in6 *) &addr) == 0);
-	addrs[0] = addr;
-
 	snprintf(ip, sizeof(ip), "0.0.0.0:%d", res);
 	spa_assert_se(parse_ipv4_address(ip, (struct sockaddr_in *) &addr) == 0);
+	addrs[0] = addr;
+
+	snprintf(ip, sizeof(ip), "[::]:%d", res);
+	spa_assert_se(parse_ipv6_address(ip, (struct sockaddr_in6 *) &addr) == 0);
 	addrs[1] = addr;
+
 	return 2;
 }
 
@@ -899,7 +903,7 @@ static struct server *server_new(struct impl *impl)
 
 static int server_start(struct server *server, const struct sockaddr_storage *addr)
 {
-	const struct impl * const impl = server->impl;
+	struct impl * const impl = server->impl;
 	int res = 0, fd;
 
 	switch (addr->ss_family) {
@@ -924,6 +928,8 @@ static int server_start(struct server *server, const struct sockaddr_storage *ad
 		res = -errno;
 		pw_log_error("server %p: can't create server source: %m", impl);
 	}
+	if (res >= 0)
+		spa_hook_list_call(&impl->hooks, struct impl_events, server_started, 0, server);
 
 	return res;
 }
@@ -1069,6 +1075,8 @@ void server_free(struct server *server)
 		client_unref(c);
 	}
 
+	spa_hook_list_call(&impl->hooks, struct impl_events, server_stopped, 0, server);
+
 	if (server->source)
 		pw_loop_destroy_source(impl->loop, server->source);
 
