@@ -50,7 +50,7 @@
 #include <spa/utils/string.h>
 #include <spa/utils/json.h>
 #include <spa/utils/ringbuffer.h>
-#include <spa/debug/pod.h>
+#include <spa/debug/types.h>
 #include <spa/pod/builder.h>
 #include <spa/param/audio/format-utils.h>
 #include <spa/param/audio/raw.h>
@@ -58,7 +58,6 @@
 
 #include <pipewire/impl.h>
 #include <pipewire/i18n.h>
-#include <pipewire/private.h>
 
 #include "module-raop/rtsp-client.h"
 
@@ -579,10 +578,10 @@ static int create_udp_socket(struct impl *impl, uint16_t *port)
 
 		if (ip_version == 4) {
 			sa4.sin_port = htons(*port);
-			ret = bind(fd, &sa4, sizeof(sa4));
+			ret = bind(fd, (struct sockaddr*)&sa4, sizeof(sa4));
 		} else {
 			sa6.sin6_port = htons(*port);
-			ret = bind(fd, &sa6, sizeof(sa6));
+			ret = bind(fd, (struct sockaddr*)&sa6, sizeof(sa6));
 		}
 		if (ret == 0)
 			break;
