@@ -35,9 +35,9 @@
 #include <spa/utils/result.h>
 #include <spa/utils/string.h>
 #include <spa/utils/json.h>
+#include <spa/param/audio/format-utils.h>
 
 #include <pipewire/impl.h>
-#include <pipewire/private.h>
 #include <pipewire/i18n.h>
 
 #include <avahi-client/lookup.h>
@@ -54,7 +54,10 @@
  * audio to/from remote PulseAudio servers. It also works with
  * module-protocol-pulse.
  *
- * This module has no options.
+ * ## Module Options
+ *
+ * - `pulse.latency`: the latency to end-to-end latency in milliseconds to
+ *                    maintain (Default 200ms).
  *
  * ## Example configuration
  *
@@ -72,7 +75,7 @@
 PW_LOG_TOPIC_STATIC(mod_topic, "mod." NAME);
 #define PW_LOG_TOPIC_DEFAULT mod_topic
 
-#define MODULE_USAGE	" "
+#define MODULE_USAGE	"pulse.latency=<latency in msec> "
 
 static const struct spa_dict_item module_props[] = {
 	{ PW_KEY_MODULE_AUTHOR, "Wim Taymans <wim.taymans@gmail.com>" },
@@ -192,7 +195,7 @@ static void pw_properties_from_avahi_string(const char *key, const char *value,
 		struct pw_properties *props)
 {
 	if (spa_streq(key, "device")) {
-		pw_properties_set(props, PW_KEY_NODE_TARGET, value);
+		pw_properties_set(props, PW_KEY_TARGET_OBJECT, value);
 	}
 	else if (spa_streq(key, "rate")) {
 		pw_properties_set(props, PW_KEY_AUDIO_RATE, value);
@@ -304,7 +307,7 @@ static void resolver_cb(AvahiServiceResolver *r, AvahiIfIndex interface, AvahiPr
 		avahi_free(value);
 	}
 
-	if ((device = pw_properties_get(props, PW_KEY_NODE_TARGET)) != NULL)
+	if ((device = pw_properties_get(props, PW_KEY_TARGET_OBJECT)) != NULL)
 		pw_properties_setf(props, PW_KEY_NODE_NAME,
 				"tunnel.%s.%s", host_name, device);
 	else
@@ -327,7 +330,7 @@ static void resolver_cb(AvahiServiceResolver *r, AvahiIfIndex interface, AvahiPr
 	if (desc == NULL)
 		desc = pw_properties_get(props, PW_KEY_DEVICE_PRODUCT_NAME);
 	if (desc == NULL)
-		desc = pw_properties_get(props, PW_KEY_NODE_TARGET);
+		desc = pw_properties_get(props, PW_KEY_TARGET_OBJECT);
 	if (desc == NULL)
 		desc = _("Unknown device");
 
@@ -348,6 +351,9 @@ static void resolver_cb(AvahiServiceResolver *r, AvahiIfIndex interface, AvahiPr
 				_("%s on %s"), desc, fqdn);
 	}
 
+	if ((str = pw_properties_get(impl->properties, "pulse.latency")) != NULL)
+		pw_properties_set(props, "pulse.latency", str);
+
 	if ((f = open_memstream(&args, &size)) == NULL) {
 		pw_log_error("Can't open memstream: %m");
 		goto done;
