@@ -36,9 +36,7 @@
 #include <spa/utils/string.h>
 #include <spa/pod/filter.h>
 #include <spa/pod/dynamic.h>
-#include <spa/debug/format.h>
 #include <spa/debug/types.h>
-#include <spa/debug/pod.h>
 
 #include "pipewire/pipewire.h"
 #include "pipewire/filter.h"
@@ -419,16 +417,16 @@ static int enum_params(struct filter *d, struct spa_list *param_list, int seq,
 	spa_list_for_each(p, param_list, link) {
 		struct spa_pod *param;
 
-		result.index = result.next++;
-		if (result.index < start)
-			continue;
-
 		param = p->param;
 		if (param == NULL || p->id != id)
 			continue;
 
 		found = true;
 
+		result.index = result.next++;
+		if (result.index < start)
+			continue;
+
 		spa_pod_dynamic_builder_init(&b, buffer, sizeof(buffer), 4096);
 		if (spa_pod_filter(&b.b, &result.param, param, filter) == 0) {
 			spa_node_emit_result(&d->hooks, seq, 0, SPA_RESULT_TYPE_NODE_PARAMS, &result);
@@ -885,16 +883,19 @@ static int impl_port_use_buffers(void *object,
 	pw_log_debug("%p: port:%d.%d buffers:%u disconnecting:%d", impl,
 			direction, port_id, n_buffers, impl->disconnecting);
 
+	if ((port = get_port(impl, direction, port_id)) == NULL)
+		return -EINVAL;
+
 	if (impl->disconnecting && n_buffers > 0)
 		return -EIO;
 
-	if ((port = get_port(impl, direction, port_id)) == NULL)
-		return -EINVAL;
+	clear_buffers(port);
 
 	impl_flags = port->flags;
 	prot = PROT_READ | (direction == SPA_DIRECTION_OUTPUT ? PROT_WRITE : 0);
 
-	clear_buffers(port);
+	if (n_buffers > MAX_BUFFERS)
+		return -ENOSPC;
 
 	for (i = 0; i < n_buffers; i++) {
 		int buf_size = 0;
