@@ -1199,8 +1199,8 @@ static void result_port_params(void *data, int seq, int res, uint32_t type, cons
 			d->callback(d->data, seq, r->id, r->index, r->next, r->param);
 			if (d->cache) {
 				if (d->count++ == 0)
-					pw_param_add(&impl->pending_list, r->id, NULL);
-				pw_param_add(&impl->pending_list, r->id, r->param);
+					pw_param_add(&impl->pending_list, seq, r->id, NULL);
+				pw_param_add(&impl->pending_list, seq, r->id, r->param);
 			}
 		}
 		break;
@@ -1253,10 +1253,10 @@ int pw_impl_port_for_each_param(struct pw_impl_port *port,
 		result.next = 0;
 
 		spa_list_for_each(p, &impl->param_list, link) {
-			result.index = result.next++;
 			if (p->id != param_id)
 				continue;
 
+			result.index = result.next++;
 			if (result.index < index)
 				continue;
 
@@ -1286,7 +1286,7 @@ int pw_impl_port_for_each_param(struct pw_impl_port *port,
 		spa_hook_remove(&listener);
 
 		if (user_data.cache) {
-			pw_param_update(&impl->param_list, &impl->pending_list);
+			pw_param_update(&impl->param_list, &impl->pending_list, 0, NULL);
 			pi->user = 1;
 		}
 	}
@@ -1473,6 +1473,12 @@ int pw_impl_port_set_param(struct pw_impl_port *port, uint32_t id, uint32_t flag
 			port->added = false;
 		}
 		/* setting the format always destroys the negotiated buffers */
+		if (port->direction == PW_DIRECTION_OUTPUT) {
+			struct pw_impl_link *l;
+			/* remove all buffers shared with an output port peer */
+			spa_list_for_each(l, &port->links, output_link)
+				pw_impl_port_use_buffers(l->input, &l->rt.in_mix, 0, NULL, 0);
+		}
 		pw_buffers_clear(&port->buffers);
 		pw_buffers_clear(&port->mix_buffers);
 
@@ -1567,6 +1573,7 @@ int pw_impl_port_use_buffers(struct pw_impl_port *port, struct pw_impl_port_mix
 		return -EIO;
 
 	if (n_buffers == 0) {
+		mix->have_buffers = false;
 		if (port->n_mix == 1)
 			pw_impl_port_update_state(port, PW_IMPL_PORT_STATE_READY, 0, NULL);
 	}
